#include "ActorsFactory.h"
#include <algorithm>
#include <stdio.h>
#include <vector>

#if OXYGINE_VERSION > 6
#include "oxygine/actor/Actor.h"
#include "oxygine/res/Resource.h"
#include "oxygine/res/Resources.h"
#include "oxygine/actor/Actor.h"
#include "oxygine/utils/stringUtils.h"
#else
#include "ox/Actor.hpp"
#include "Resource.h"
#include "Resources.h"
#include "ox/Actor.hpp"
#include "stringUtils.h"
#endif

using namespace std;

namespace oxygine
{
    bool _needSort = false;
    struct actor
    {
        createActorCallback cb;
        associateNodeCallback an;
        const char* name;

        static bool cmp(const actor& s1, const actor& s2)
        {
            return strcmp(s1.name, s2.name) > 0;
        }
    };


    vector<actor>& getFactoryActors()
    {
        static vector<actor> vec;
        return vec;
    }

    bool registerFactoryActor(createActorCallback cb, associateNodeCallback an, const char* name)
    {
        _needSort = true;
        actor s;
        s.cb = cb;
        s.an = an;
        s.name = name;
        getFactoryActors().push_back(s);

        return true;
    }


    bool findActorPred(const actor& s, const char* name)
    {
        return strcmp(s.name, name) > 0;
    }


    vector<actor>::iterator findFactoryActor(const char* name, bool showError)
    {
        vector<actor>& items = getFactoryActors();
        if (_needSort)
        {
            sort(items.begin(), items.end(), actor::cmp);
            _needSort = false;
        }

        vector<actor>::iterator i = lower_bound(items.begin(), items.end(), name, findActorPred);
        if ((i == items.end() || strcmp(i->name, name)))
        {
            if (showError)
            {
                logs::error("can't find factory class: %s", name);
                OX_ASSERT(!"can't find factory class");
            }
            return items.end();
        }

        return i;
    }


    Actor* createFactoryActor(const char* name, const creator* d)
    {
        vector<actor>::iterator i = findFactoryActor(name, true);
        if (i == getFactoryActors().end())
        {
            return 0;
        }

        const actor& c = *i;
        Actor* s = c.cb(d);
        return s;
    }

    void associateFactoryActorNode(const char* name, pugi::xml_node node)
    {
        vector<actor>::iterator i =  findFactoryActor(name, node.attribute("gen_code").as_bool());
        if (i == getFactoryActors().end())
        {
            return;
        }

        const actor& c = *i;
        c.an(node);
    }

    pugi::xml_node getActorClassNode(const pugi::xml_document& doc, const char* name)
    {
        pugi::xml_node node = doc.document_element().child("class");
        pugi::xml_node child = node.first_child();
        while (child)
        {
            if (!strcmp(child.attribute("class").as_string(), name))
                return child;
            child = child.next_sibling();
        }
        return node;
    }

    void associateActorClasses(const pugi::xml_document& doc)
    {
        pugi::xml_node node = doc.document_element().child("class").first_child();
        while (node)
        {
            const  char* className = node.attribute("class").as_string();
            associateFactoryActorNode(className, node);
            node = node.next_sibling();
        }
    }


    void factory::clear()
    {
        _parent = 0;
        _res.clear();
    }

    void factory::add(Resources* r)
    {
        _res.push_back(r);
    }

    spActor factory::create(const char* type) const
    {
        spActor actor = creator::create(type);
        if (actor)
            return actor;

        actor = createFactoryActor(type, this);
        return actor;
    }

    Resource* factory::getResource(const char* name) const
    {
        char id[255];
        char ext[64];

        path::splitExt(name, id, ext);

        for (size_t i = 0; i < _res.size(); ++i)
        {
            Resources* r = _res[i];
            Resource* res = r->get(id, ep_show_warning);
            if (res)
                return res;
        }

        if (_parent)
            return _parent->getResource(name);

        return 0;
    }

    Actor* ActorX::_findDescendantX(Actor* parent, const char* name)
    {
        if (parent->isName(name))
            return parent;
        if (_getFlags(parent) & flag_reserved)
            return 0;

        Actor* child = parent->getFirstChild().get();
        while (child)
        {
            Actor* des = _findDescendantX(child, name);
            if (des)
                return des;

            child = child->getNextSibling().get();
        }

        return 0;
    }

    Actor* ActorX::_findDescendant(const char* name)
    {
        OX_ASSERT(_flags & flag_reserved);
        _flags &= ~flag_reserved;
        Actor* res = _findDescendantX(this, name);
        _flags |= flag_reserved;
        //OX_ASSERT(res != null);
        return res;
    }


    spActor createActor(const factory& factory, const pugi::xml_document& doc, const string& name)
    {
        pugi::xml_node node = doc.document_element().child("class").first_child();
        while (node)
        {
            const  char* className = node.attribute("class").as_string();
            if (name == className)
                break;
            node = node.next_sibling();
        }


        spActor view = new Actor;
        deserializedata d;
        d.node = node;
        d.factory = &factory;
        view->deserialize(&d);

        deserializeLinkData link;
        link.root = view.get();
        link.node = d.node;
        view->deserializeLink(&link);
        link.root = 0;

        return view;
    }
}