#include "oxygine-framework.h"
#include "ActorsFactory.h"

#include "gview/MainView.h"
#include "gview/World.h"
#include "gview/DialogView.h"
#include "gview/UiView.h"
#include "gview/MyButton.h"
using namespace oxygine;


void example_preinit()
{

}

factory _factory;
Resources _res;
pugi::xml_document _doc;

spMainView _mainView;

void example_init()
{
    DebugActor::setCorner(2);

    //load resources from xml with images and fonts
    _res.loadXML("xml/res.xml");
    _res.loadXML("xml/fonts.xml");

    //add it to factory
    _factory.add(&_res);


    //load and parse xml document with created Scene in editor from file
    file::buffer bf;
    file::read("xml/scene.oxv", bf);
    _doc.load_buffer(bf.getData(), bf.getSize());


    //associates data from xml with generated classes
    associateActorClasses(_doc);

    //create our class
    spMainView m = new MainView(_factory);//factory as argument


    m->world->earth->addTween(Actor::TweenRotationDegrees(360), 6000, -1);
    m->world->earth->addClickListener([ = ](Event*)
    {
        m->world->moon->addTween(Actor::TweenRotationDegrees(m->world->moon->getRotationDegrees() + 360), 2000, 1);
    });

    m->world->sun->addTween(Actor::TweenRotationDegrees(360), 5000, -1);

    m->world->planet1->addTween(Actor::TweenRotationDegrees(360), 7000, -1);
    m->world->planet2->addTween(Actor::TweenRotationDegrees(360), 8000, -1);
    m->world->planet3->addTween(Actor::TweenRotationDegrees(360), 10000, -1);

    m->ui->btn->txt->setText("Menu");


    m->ui->btn->addEventListener(TouchEvent::CLICK, [](Event*)
    {
        spDialogView dialog = new DialogView(_factory);
        dialog->btn->txt->setText("Close");
        dialog->attachTo(getStage());
        dialog->setPosition(getStage()->getSize() / 2 - dialog->getSize() / 2);
        dialog->btn->addEventListener(TouchEvent::CLICK, [ = ](Event*)
        {
            dialog->detach();
            dialog->btn->removeAllEventListeners();
        });
    });

    //add it to stage
    m->attachTo(getStage());

    _mainView = m;
}

void example_update()
{

}

void example_destroy()
{
    _res.free();
    //_mainView was captured by lambda
    //from m->world->earth->addClickListener
    //and it won't be automatically deleted until we remove event listener with capture
    _mainView->world->earth->removeAllEventListeners();
    _mainView = 0;
}